@@ -3,7 +3,7 @@ import token
 import functools
 from typing import List, Type, Tuple, TypeVar, Optional
 
-import asttokens.util  # type: ignore[import]
+import asttokens.util
 from asttokens import ASTTokens
 
 from .exceptions import TuckError
@@ -15,14 +15,14 @@ def _first_token(node: ast.AST) -> asttokens.util.Token:
     if isinstance(node, ast.GeneratorExp):
         return _first_token(node.elt)
 
-    return node.first_token  # type: ignore[attr-defined]
+    return node.first_token  # type: ignore[attr-defined,no-any-return]
 
 
 def _last_token(node: ast.AST) -> asttokens.util.Token:
     if isinstance(node, ast.GeneratorExp):
         return _last_token(node.generators[-1])
 
-    return node.last_token  # type: ignore[attr-defined]
+    return node.last_token  # type: ignore[attr-defined,no-any-return]
 
 
 @functools.total_ordering
@@ -53,7 +53,7 @@ class Position:
 
     def __lt__(self, other: 'Position') -> bool:
         if not isinstance(other, Position):
-            return NotImplemented  # type: ignore  # unreachable
+            return NotImplemented  # type: ignore[unreachable]
 
         if self.line < other.line:
             return True
@@ -99,6 +99,10 @@ class NoSupportedNodeFoundError(NodeSearchError):
 
 
 class NodeFinder(ast.NodeVisitor):
+    """
+    Visitor which finds the AST node that should be wrapped for a given position.
+    """
+
     def __init__(self, position: Position, node_types: Tuple[Type[ast.AST], ...]) -> None:
         self.target_position = position
         self.target_node_types = node_types
@@ -170,6 +174,27 @@ class NodeFinder(ast.NodeVisitor):
 
         raise NoSupportedNodeFoundError(self.node_stack)
 
+    def _get_end_of_node(self, node: ast.AST) -> Position:
+        # We want to prefer the node that the position is clearly "within". This
+        # means slightly different things for different cases:
+        #  - nodes that end with some for of bracket don't really want to
+        #    consider the bracket as in practise text editors position their
+        #    visible cursor to the left of the character position they're at
+        #  - generators sort-of end with a bracket, but don't own the bracket
+        #  - due to how Python's AST is structured around attributes and calls,
+        #    we don't want to consider the right hand "name" portion of an
+        #    attribute as part of the attribute; this is so we're forced to
+        #    consider only the parent node rather than a true reflection of the
+        #    positions
+        last_token = _last_token(node)
+        if (
+            last_token.type == token.OP and last_token.string in ')}]'
+            or isinstance(node, (ast.Attribute, ast.Name, ast.GeneratorExp))
+        ):
+            return Position(*last_token.start)
+
+        return Position(*last_token.end)
+
     def generic_visit(self, node: ast.AST) -> None:
         if self.found:
             return
@@ -179,7 +204,7 @@ class NodeFinder(ast.NodeVisitor):
             return
 
         start = Position.from_node_start(node)
-        end = Position.from_node_end(node)
+        end = self._get_end_of_node(node)
 
         if end < self.target_position:
             # we're clear before the target
@@ -206,4 +231,51 @@ def get_current_indent(asttokens: ASTTokens, node: ast.AST) -> int:
         next_tok = tok
         tok = asttokens.prev_token(tok)
 
-    return next_tok.start[1]  # type: ignore
+    return next_tok.start[1]  # type: ignore[no-any-return]
+
+
+class AffectedNodeFinder(ast.NodeVisitor):
+    """
+    Visitor which finds the first AST node that would be affected by wrapping at
+    the given position.
+
+    Nodes which are all on the same line will never be considered matches.
+    """
+
+    def __init__(self, position: Position) -> None:
+        self.target_position = position
+
+        self._found: bool = False
+        self.found_node: Optional[ast.AST] = None
+
+    def generic_visit(self, node: ast.AST) -> None:
+        if self._found:
+            return
+
+        if not hasattr(node, 'lineno'):
+            super().generic_visit(node)
+            return
+
+        start = Position.from_node_start(node)
+        end = Position.from_node_end(node)
+
+        if end < self.target_position:
+            # we're clear before the target
+            return
+
+        if start.line > self.target_position.line:
+            # we're clear after the target
+            return
+
+        if start.line == end.line:
+            # not interested in single-line nodes
+            return
+
+        if (
+            start.line == self.target_position.line
+            and start.col >= self.target_position.col
+        ):
+            self.found_node = node
+            return
+
+        super().generic_visit(node)
