@@ -1,7 +1,7 @@
 import ast
 from typing import List, Tuple, TypeVar, Optional
 
-from asttokens import ASTTokens  # type: ignore[import]
+from asttokens import ASTTokens
 
 from .ast import Position, NodeFinder, get_current_indent
 from .editing import (
@@ -13,12 +13,12 @@ from .editing import (
     merge_insertions,
     indent_interim_lines,
 )
-from .wrappers import WRAPPING_FUNTIONS
+from .wrappers import WRAPPING_FUNCTIONS
 from .exceptions import TuckError
 
 TAst = TypeVar('TAst', bound=ast.AST)
 
-WRAPPABLE_NODE_TYPES = tuple(x for x, _ in WRAPPING_FUNTIONS)
+WRAPPABLE_NODE_TYPES = tuple(x for x, _ in WRAPPING_FUNCTIONS)
 
 
 class TargetSyntaxError(TuckError):
@@ -27,7 +27,7 @@ class TargetSyntaxError(TuckError):
 
 
 def get_wrapping_summary(asttokens: ASTTokens, node: ast.AST) -> WrappingSummary:
-    for ast_type, func in WRAPPING_FUNTIONS:
+    for ast_type, func in WRAPPING_FUNCTIONS:
         if isinstance(node, ast_type):
             return func(asttokens, node)
 
@@ -123,6 +123,7 @@ def remove_redundant_wrapping_operations(
 
 def determine_insertions(asttokens: ASTTokens, position: Position) -> List[Insertion]:
     finder = NodeFinder(position, WRAPPABLE_NODE_TYPES)
+    assert asttokens.tree is not None
     finder.visit(asttokens.tree)
 
     node = finder.found_node
@@ -143,14 +144,18 @@ def determine_insertions(asttokens: ASTTokens, position: Position) -> List[Inser
 
     wrapping_summary = get_wrapping_summary(asttokens, node)
 
-    wrapping_summary = indent_interim_lines(asttokens, wrapping_summary)
-
     wrapping_summary = remove_redundant_wrapping_operations(
         asttokens,
         wrapping_summary,
         Position.from_node_start(node),
     )
 
+    wrapping_summary = indent_interim_lines(
+        asttokens,
+        wrapping_summary,
+        root=node,
+    )
+
     insertions = [
         (pos, ''.join(mutations[x] for x in mutation_types))
         for pos, mutation_types in coalesce(wrapping_summary)
